/*------------------------------------------------------------------------
  File  : rtb/jira/IssueQuery.cls
  Desc  : Jira Issues Query
  Notes : Create an instance, set properties, and fetch
    
    Example Usage: 
    
    {ttIssues.i}

    issueQuery = NEW IssueQuery().

    ASSIGN
      issueQuery:assignee    = "bob"
      issueQuery:projectList = "sports2020"
      issueQuery:typeList    = "bug"
      
    issueQuery:fetchIssues(TABLE ttIssues BY-REFERENCE).        
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.
USING OpenEdge.Net.HTTP.*.
USING Progress.Json.ObjectModel.*.
USING rtb.jira.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS rtb.jira.IssueQuery INHERITS JiraData:
  
  {rtb/jira/i/ttIssue.i &reference-only=REFERENCE-ONLY}
  
  DEFINE PUBLIC PROPERTY projectList     AS CHARACTER GET. SET.
  DEFINE PUBLIC PROPERTY issueAssignee   AS CHARACTER GET. SET.
  DEFINE PUBLIC PROPERTY issueTypeList   AS CHARACTER GET. SET.
  DEFINE PUBLIC PROPERTY openIssuesOnly  AS LOGICAL   GET. SET. 

  CONSTRUCTOR IssueQuery ():
    SUPER().
  END CONSTRUCTOR.
  
  METHOD PRIVATE CHARACTER buildJqlQuery ():
  
    DEFINE VARIABLE cFieldList AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cJql       AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cSort      AS CHARACTER NO-UNDO.
    
    ASSIGN
      cFieldList = "id,key,description,issuetype,assignee,status,summary,priority"
      cSort      = "+order+by+updated+DESC".
    
    cJql = "search/?jql="
      + (IF projectList <> "" THEN "project in(" + projectList + ")" ELSE "")
      + (IF issueAssignee   <> "" THEN " and assignee=" + QUOTER(issueAssignee) ELSE "")
      + (IF issueTypeList   <> "" THEN " and issueType in(" + issueTypeList + ")"  ELSE "")
      + (IF openIssuesOnly THEN " and resolution=unresolved" ELSE "")
      + cSort
      + (IF cFieldList <> ""   THEN " &fields="       + cFieldList ELSE "").
      
    RETURN cJql.  
    
  END METHOD.
  
  METHOD PUBLIC VOID fetchIssues (INPUT TABLE ttIssue):
    /*
     Parameters:
       INPUT - ttIsssues temp-table to store issues.
    */    
    DEFINE VARIABLE issueData          AS JsonObject     NO-UNDO.
    DEFINE VARIABLE issue              AS JsonObject     NO-UNDO.
    DEFINE VARIABLE issueType          AS JsonObject     NO-UNDO.
    DEFINE VARIABLE issueFields        AS JsonObject     NO-UNDO.
    DEFINE VARIABLE issueAssignee      AS JsonObject     NO-UNDO.
    DEFINE VARIABLE issueStatus        AS JsonObject     NO-UNDO.
    DEFINE VARIABLE issuePriority      AS JsonObject     NO-UNDO.
    DEFINE VARIABLE issueArray         AS JsonArray      NO-UNDO.

    DEFINE VARIABLE cGet  AS CHARACTER NO-UNDO.
    DEFINE VARIABLE cJql  AS CHARACTER NO-UNDO.
    DEFINE VARIABLE i     AS INTEGER   NO-UNDO.
    DEFINE VARIABLE iCnt  AS INTEGER   NO-UNDO.
    
    ASSIGN 
      cJql = buildJqlQuery()
      cGet = jiraConfig:jiraUrl + cJql.
      
    getRequest(cGet).

    IF httpResponse:StatusCode <> INTEGER(StatusCodeEnum:OK) THEN
      UNDO, THROW NEW AppError ("Could not fetch Issues.", httpResponse:StatusCode).
        
    IF TYPE-OF(httpResponse:Entity, JsonObject) THEN
      issueData = CAST(httpResponse:Entity, JsonObject).
        
    ASSIGN
      iCnt       = issueData:GetInteger("total")  
      issueArray = issueData:GetJsonArray("issues").
            
    DO i = 1 TO iCnt:
          
      ASSIGN 
        issue         = issueArray:GetJsonObject(i)
        issueFields   = issue:GetJsonObject("fields")
        issueType     = issueFields:GetJsonObject("issuetype")
        issueAssignee = issueFields:GetJsonObject("assignee")
        issueStatus   = issueFields:GetJsonObject("status")
        issuePriority = issueFields:GetJsonObject("priority").
              
      CREATE ttIssue.
      ASSIGN 
        ttIssue.issueId          = issue:GetCharacter("id")
        ttIssue.issueKey         = issue:GetCharacter("key")
        ttIssue.issueSummary     = issueFields:GetCharacter("summary")
        ttIssue.issueType        = issueType:GetCharacter("name")
        ttIssue.issueAssignee    = issueAssignee:GetCharacter("displayName")
        ttIssue.issueDescription = issueFields:GetCharacter("description") /* v2 API - must remove for v3 */
        ttIssue.issueStatus      = issueStatus:GetCharacter("name")
        ttIssue.issuePriority    = issuePriority:GetCharacter("name")
        NO-ERROR.
      
      /*
       For the v3 API, the description is more complicated as it has formatted tags in the JSON data.
       Example below for fetching paragraph data and putting into a text field. Other tags such as
       orderedlist and bulletlist need to be coded.
      */
      
      /*
      ASSIGN 
        issueDescription = issueFields:GetJsonObject("description")
        issueDescArray   = issueDescription:GetJsonArray("content").
      
      DO iCnt2 = 1 TO issueDescArray:LENGTH:
        IF issueDescArray:GetJsonObject(iCnt2):GetCharacter("type")  = "paragraph" THEN 
          ttIssue.issueDescription = ttIssue.issueDescription
                                   + (IF ttIssue.issueDescription <> "" THEN "~n" ELSE "") 
                                   + issueDescArray:GetJsonObject(iCnt2):GetJsonArray("content"):GetJsonObject(1):GetCharacter("text").
      END.
      */
      
    END.         

    RETURN.
    
    FINALLY:
      /*
       I don't necessarily trust garbage collection
      */
      DELETE OBJECT issueData     NO-ERROR.
      DELETE OBJECT issue         NO-ERROR.
      DELETE OBJECT issueType     NO-ERROR.
      DELETE OBJECT issueFields   NO-ERROR.
      DELETE OBJECT issueAssignee NO-ERROR.
      DELETE OBJECT issueStatus   NO-ERROR.
      DELETE OBJECT issuePriority NO-ERROR.
      DELETE OBJECT issueArray    NO-ERROR.
    END FINALLY.
    
  END METHOD.

END CLASS.
