/*------------------------------------------------------------------------
  File   : WorkspaceProjectMap.cls
  Notes  : Map Roundtable Workspaces to Jira projects via pre-defined
           JSON map file.
  Example:
           
    [
      {
      "workspace":"devel",
      "projects":[{"project":"sports2000"}, {"project":"sports2020"}]
      },
      {
      "workspace":"test",
      "projects":[{"project":"sports2000"}]
      } 
    ]
 ----------------------------------------------------------------------*/
USING Progress.Lang.*.
USING Progress.Json.ObjectModel.*.
USING rtb.jira.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS rtb.jira.ProjectMap:
  
  &SCOPED-DEFINE MAP-FILE "projectmap.json"
  
  DEFINE TEMP-TABLE ttMap
    FIELD workspace AS CHARACTER 
    FIELD project   AS CHARACTER.
 
  CONSTRUCTOR ProjectMap():
    loadMap().  
  END CONSTRUCTOR.
  
  METHOD PUBLIC CHARACTER EXTENT getProjects (pcWorkspace AS CHARACTER):
    
    DEFINE VARIABLE cProjectList AS CHARACTER EXTENT NO-UNDO.
    
    DEFINE QUERY qryMap FOR ttMap.
    
    OPEN QUERY qryMap PRESELECT EACH ttMap WHERE ttMap.workspace = pcWorkspace.
    IF QUERY qryMap:NUM-RESULTS > 0 THEN
      EXTENT(cProjectList) = QUERY qryMap:NUM-RESULTS.
    
    GET FIRST qryMap.
    DO WHILE AVAILABLE ttMap:
      cProjectList[QUERY qryMap:CURRENT-RESULT-ROW ] = ttMap.project.
      GET NEXT qryMap.
    END.
    
    RETURN cProjectList.
    
    FINALLY:
      CLOSE QUERY qryMap.
    END FINALLY.
    
  END METHOD. 
  
  METHOD PRIVATE VOID loadMap():
    
    DEFINE VARIABLE workspaceProjects AS JsonArray         NO-UNDO.
    DEFINE VARIABLE workspace         AS JsonObject        NO-UNDO.
    DEFINE VARIABLE projects          AS JsonArray         NO-UNDO.
    DEFINE VARIABLE project           AS JsonObject        NO-UNDO.
    DEFINE VARIABLE parser            AS ObjectModelParser NO-UNDO.
    DEFINE VARIABLE construct         AS JsonConstruct     NO-UNDO.
    
    DEFINE VARIABLE cMap AS CHARACTER NO-UNDO.
    DEFINE VARIABLE i    AS INTEGER   NO-UNDO.
    DEFINE VARIABLE j    AS INTEGER   NO-UNDO.
    
    cMap = SEARCH("jira~/conf~/" + {&MAP-FILE}).
    
    ASSIGN 
      parser            = NEW ObjectModelParser( ).
      construct         = parser:ParseFile(cMap).
      workspaceProjects = CAST(construct, JsonArray).
      
    DO i = 1 TO workspaceProjects:LENGTH:
      
      workspace = workspaceProjects:GetJsonObject(i).
      projects  = workspace:GetJsonArray("projects").
      
      DO j = 1 TO projects:LENGTH:
        project = projects:GetJsonObject(j).
        
        CREATE ttMap.
        ASSIGN 
          ttMap.workspace = workspace:GetCharacter("workspace")
          ttMap.project   = QUOTER(project:GetCharacter("project")).
            
      END.
      
    END.  
    
    RETURN.
    
    FINALLY:
      DELETE OBJECT workspaceProjects NO-ERROR.
      DELETE OBJECT workspace         NO-ERROR.
      DELETE OBJECT projects          NO-ERROR.
      DELETE OBJECT project           NO-ERROR.
      DELETE OBJECT parser            NO-ERROR.
      DELETE OBJECT construct         NO-ERROR.
    END FINALLY.
    
  END METHOD.
  
END CLASS.
